@@ -1,3 +1,18 @@
+
+
+/**
+DEFAULT ALSA PARAMETERS CONTROLLED BY ENVIRONMENT VARIABLES
+
+Some default parameters of Faust's ALSA applications are controlled by the following environment variables :
+
+    FAUST2ALSA_DEVICE   = "hw:0"
+    FAUST2ALSA_FREQUENCY= 44100
+    FAUST2ALSA_BUFFER   = 1024
+    FAUST2ALSA_PERIODS  = 2
+
+*/
+
+
 /* link with : "" */
 #include <stdlib.h>
 #include <libgen.h>
@@ -17,6 +32,8 @@
 #include <pthread.h> 
 #include <sys/wait.h>
 #include <list>
+#include <map>
+#include <set>
 
 #include <iostream>
 #include <fstream>
@@ -38,6 +55,81 @@
 //inline int lsr (int x, int n)		{ return int(((unsigned int)x) >> n); }
 //inline int int2pow2 (int x)		{ int r=0; while ((1<<r)<x) r++; return r; }
 
+using namespace std;
+
+// On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
+// flags to avoid costly denormals
+#ifdef __SSE__
+    #include <xmmintrin.h>
+    #ifdef __SSE2__
+        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8040)
+    #else
+        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8000)
+    #endif
+#else
+    #define AVOIDDENORMALS 
+#endif
+
+
+//#define BENCHMARKMODE
+
+#ifdef BENCHMARKMODE
+// mesuring jack performances
+static __inline__ unsigned long long int rdtsc(void)
+{
+  unsigned long long int x;
+     __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
+     return x;
+}
+
+#define KSKIP 10
+#define KMESURE 1024
+int mesure = 0;
+unsigned long long int starts[KMESURE];
+unsigned long long int stops [KMESURE];
+
+#define STARTMESURE starts[mesure%KMESURE] = rdtsc();
+#define STOPMESURE stops[mesure%KMESURE] = rdtsc(); mesure = mesure+1;
+
+void printstats()
+{
+    unsigned long long int low, hi, tot;
+    low = hi = tot = (stops[KSKIP] - starts[KSKIP]);
+
+    if (mesure < KMESURE) {
+    
+        for (int i = KSKIP+1; i<mesure; i++) {
+            unsigned long long int m = stops[i] - starts[i];
+            if (m<low) low = m;
+            if (m>hi) hi = m;
+            tot += m;
+        }
+        cout << low << ' ' << tot/(mesure-KSKIP) << ' ' << hi << endl;
+
+    } else {
+    
+        for (int i = KSKIP+1; i<KMESURE; i++) {
+            unsigned long long int m = stops[i] - starts[i];
+            if (m<low) low = m;
+            if (m>hi) hi = m;
+            tot += m;
+        }
+        cout << low << ' ' << tot/(KMESURE-KSKIP) << ' ' << hi << endl;
+
+    }    
+}
+#else
+
+#define STARTMESURE
+#define STOPMESURE
+
+#endif
+
+struct Meta : map<const char*, const char*>
+{
+    void declare (const char* key, const char* value) { (*this)[key]=value; }
+};
+
 
 /**
  * Used to set the priority and scheduling of the audio thread 
@@ -68,7 +160,7 @@ bool setRealtimePriority ()
 *******************************************************************************/
 
 //inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((unsigned)(calloc((nmemb*size)+15,sizeof(char)))+15 & 0xfffffff0); }
-inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((size_t)(calloc((nmemb*size)+15,sizeof(char)))+15 & ~15); }
+//inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((size_t)(calloc((nmemb*size)+15,sizeof(char)))+15 & ~15); }
 
 
 <<includeIntrinsic>>
@@ -95,6 +187,7 @@ class AudioParam
 	const char*		fCardName;					
 	unsigned int	fFrequency;
 	int				fBuffering; 
+    unsigned int    fPeriods; 
 	
 	unsigned int	fSoftInputs;
 	unsigned int	fSoftOutputs;
@@ -104,6 +197,7 @@ class AudioParam
 		fCardName("hw:0"),
 		fFrequency(44100),
 		fBuffering(512),
+        fPeriods(2),
 		fSoftInputs(2),
 		fSoftOutputs(2)
 	{}
@@ -111,6 +205,7 @@ class AudioParam
 	AudioParam&	cardName(const char* n)	{ fCardName = n; 		return *this; }
 	AudioParam&	frequency(int f)		{ fFrequency = f; 		return *this; }
 	AudioParam&	buffering(int fpb)		{ fBuffering = fpb; 	return *this; }
+    AudioParam& periods(int p)          { fPeriods = p;         return *this; }
 	AudioParam&	inputs(int n)			{ fSoftInputs = n; 		return *this; }
 	AudioParam&	outputs(int n)			{ fSoftOutputs = n; 	return *this; }
 };
@@ -153,6 +248,7 @@ class AudioInterface : public AudioParam
 	const char*	cardName()				{ return fCardName;  	}
  	int			frequency()				{ return fFrequency; 	}
 	int			buffering()				{ return fBuffering;  	}
+    int         periods()               { return fPeriods;      }
 	
 	float**		inputSoftChannels()		{ return fInputSoftChannels;	}
 	float**		outputSoftChannels()	{ return fOutputSoftChannels;	}
@@ -181,14 +277,20 @@ class AudioInterface : public AudioParam
 		// search for optimal input parameters
 		err = snd_pcm_hw_params_malloc	( &fInputParams ); 	check_error(err);
 		setAudioParams(fInputDevice, fInputParams);
-		snd_pcm_hw_params_get_channels(fInputParams, &fCardInputs);
 
 		// search for optimal output parameters
 		err = snd_pcm_hw_params_malloc	( &fOutputParams ); 		check_error(err)
 		setAudioParams(fOutputDevice, fOutputParams);
-		snd_pcm_hw_params_get_channels(fOutputParams, &fCardOutputs);
 
-		printf("inputs : %ud, outputs : %ud\n", fCardInputs, fCardOutputs);
+		// set the number of physical input and output channels close to what we need
+		fCardInputs 	= fSoftInputs;
+		fCardOutputs 	= fSoftOutputs;
+		
+		snd_pcm_hw_params_set_channels_near(fInputDevice, fInputParams, &fCardInputs);
+		snd_pcm_hw_params_set_channels_near(fOutputDevice, fOutputParams, &fCardOutputs);
+
+		printf("inputs : %u, outputs : %u\n", fCardInputs, fCardOutputs);
+
 
 		// setup input-output parameters
 		err = snd_pcm_hw_params (fInputDevice,  fInputParams );	 	check_error (err);
@@ -198,15 +300,15 @@ class AudioInterface : public AudioParam
 
 		// allocation of alsa buffers
 		if (fSampleAccess == SND_PCM_ACCESS_RW_INTERLEAVED) {
-			fInputCardBuffer = aligned_calloc(interleavedBufferSize(fInputParams), 1);
-	 		fOutputCardBuffer = aligned_calloc(interleavedBufferSize(fOutputParams), 1);
+			fInputCardBuffer = calloc(interleavedBufferSize(fInputParams), 1);
+	 		fOutputCardBuffer = calloc(interleavedBufferSize(fOutputParams), 1);
 			
 		} else {
 			for (unsigned int i = 0; i < fCardInputs; i++) {
-				fInputCardChannels[i] = aligned_calloc(noninterleavedBufferSize(fInputParams), 1);
+				fInputCardChannels[i] = calloc(noninterleavedBufferSize(fInputParams), 1);
 			}
 			for (unsigned int i = 0; i < fCardOutputs; i++) {
-				fOutputCardChannels[i] = aligned_calloc(noninterleavedBufferSize(fOutputParams), 1);
+				fOutputCardChannels[i] = calloc(noninterleavedBufferSize(fOutputParams), 1);
 			}
 			
 		}
@@ -217,14 +319,14 @@ class AudioInterface : public AudioParam
 		fChanOutputs = max(fSoftOutputs, fCardOutputs);		assert (fChanOutputs < 256);
 
 		for (unsigned int i = 0; i < fChanInputs; i++) {
-			fInputSoftChannels[i] = (float*) aligned_calloc (fBuffering, sizeof(float));
+			fInputSoftChannels[i] = (float*) calloc (fBuffering, sizeof(float));
 			for (int j = 0; j < fBuffering; j++) {
 				fInputSoftChannels[i][j] = 0.0;
 			}
 		}
 
 		for (unsigned int i = 0; i < fChanOutputs; i++) {
-			fOutputSoftChannels[i] = (float*) aligned_calloc (fBuffering, sizeof(float));
+			fOutputSoftChannels[i] = (float*) calloc (fBuffering, sizeof(float));
 			for (int j = 0; j < fBuffering; j++) {
 				fOutputSoftChannels[i][j] = 0.0;
 			}
@@ -266,7 +368,7 @@ class AudioInterface : public AudioParam
 		err = snd_pcm_hw_params_set_period_size	(stream, params, fBuffering, 0); 	
 		check_error_msg(err, "period size not available");
 		
-		err = snd_pcm_hw_params_set_periods (stream, params, 2, 0); 			
+		err = snd_pcm_hw_params_set_periods (stream, params, fPeriods, 0);
 		check_error_msg(err, "number of periods not available");
 
 	}
@@ -571,7 +673,7 @@ mydsp	DSP;
 *******************************************************************************/
 	
 ///< lopt : Scan Command Line long int Arguments
-long lopt (int argc, char *argv[], char* longname, char* shortname, long def) 
+long lopt (int argc, char *argv[], const char* longname, const char* shortname, long def) 
 {
 	for (int i=2; i<argc; i++) 
 		if ( strcmp(argv[i-1], shortname) == 0 || strcmp(argv[i-1], longname) == 0 ) 
@@ -580,7 +682,7 @@ long lopt (int argc, char *argv[], char* longname, char* shortname, long def)
 }
 	
 ///< sopt : Scan Command Line string Arguments
-char* sopt (int argc, char *argv[], char* longname, char* shortname, char* def) 
+const char* sopt (int argc, char *argv[], const char* longname, const char* shortname, const char* def) 
 {
 	for (int i=2; i<argc; i++) 
 		if ( strcmp(argv[i-1], shortname) == 0 || strcmp(argv[i-1], longname) == 0 ) 
@@ -589,7 +691,7 @@ char* sopt (int argc, char *argv[], char* longname, char* shortname, char* def)
 }
 	
 ///< fopt : Scan Command Line flag option (without argument), return true if the flag
-bool fopt (int argc, char *argv[], char* longname, char* shortname) 
+bool fopt (int argc, char *argv[], const char* longname, const char* shortname) 
 {
 	for (int i=1; i<argc; i++) 
 		if ( strcmp(argv[i], shortname) == 0 || strcmp(argv[i], longname) == 0 ) 
@@ -607,7 +709,12 @@ bool fopt (int argc, char *argv[], char* longname, char* shortname)
 *******************************************************************************
 *******************************************************************************/
 	
-list<UI*> 	UI::fGuiList;
+list<UI*> 	            UI::fGuiList;
+map<float*, float>      QTGUI::fGuiSize;       // map widget zone with widget size coef
+map<float*, string>     QTGUI::fTooltip;       // map widget zone with tooltip strings
+map<float*, string>     QTGUI::fUnit;           // map widget zone with unit strings (i.e. "dB")
+set<float*>             QTGUI::fKnobSet;       // set of widget zone to be knobs
+set<float*>             QTGUI::fLedSet;        // set of widget zone to be LEDs
 
 UI* 		interface;
 
@@ -628,6 +735,7 @@ pthread_t	audiothread;
  */
 void* run_audio(void* ptr)
 {
+    AVOIDDENORMALS;
 	AudioInterface*	audio = (AudioInterface*) ptr;
 
 	bool rt = setRealtimePriority();
@@ -637,7 +745,9 @@ void* run_audio(void* ptr)
 	audio->write();
 	while(!interface->stopped()) {
 		audio->read();
+    STARTMESURE
 		DSP.compute(audio->buffering(), audio->inputSoftChannels(), audio->outputSoftChannels());
+    STOPMESURE
 		audio->write();
 	} 
 
@@ -646,6 +756,31 @@ void* run_audio(void* ptr)
 }
 
 
+/**
+ * Return the value of an environment variable or defval if undefined.
+ */
+static int getDefaultEnv(const char* name, int defval)
+{
+    const char* str = getenv(name);
+    if (str) {
+        return atoi(str);
+    } else {
+        return defval;
+    }
+}
+
+/**
+ * Return the value of an environment variable or defval if undefined.
+ */
+static const char* getDefaultEnv(const char* name, const char* defval)
+{
+    const char* str = getenv(name);
+    if (str) {
+        return str;
+    } else {
+        return defval;
+    }
+}
 
 /******************************************************************************
 *******************************************************************************
@@ -668,14 +803,15 @@ int main(int argc, char *argv[] )
 	char	rcfilename[256];
 	char* 	home = getenv("HOME");
 	snprintf(rcfilename, 255, "%s/.%src", home, basename(argv[0]));
-	
-	AudioInterface	audio (
-		AudioParam().cardName( sopt(argc, argv, "--device", "-d", "hw:0") ) 
-					.frequency( lopt(argc, argv, "--frequency", "-f", 44100) ) 
-					.buffering( lopt(argc, argv, "--buffer", "-b", 1024) )
-					.inputs(DSP.getNumInputs())
-					.outputs(DSP.getNumOutputs())
-	);
+
+    AudioInterface  audio (
+        AudioParam().cardName( sopt(argc, argv, "--device", "-d",     getDefaultEnv("FAUST2ALSA_DEVICE", "hw:0")  ) )
+                    .frequency( lopt(argc, argv, "--frequency", "-f", getDefaultEnv("FAUST2ALSA_FREQUENCY",44100) ) ) 
+                    .buffering( lopt(argc, argv, "--buffer", "-b",    getDefaultEnv("FAUST2ALSA_BUFFER",1024)     ) )
+                    .periods( lopt(argc, argv, "--periods", "-p",     getDefaultEnv("FAUST2ALSA_PERIODS",2)       ) )
+                    .inputs(DSP.getNumInputs())
+                    .outputs(DSP.getNumOutputs())
+    );
 
 	
 	audio.open();
@@ -691,5 +827,9 @@ int main(int argc, char *argv[] )
 	interface->run();
 	interface->saveState(rcfilename);
 
+#ifdef BENCHMARKMODE
+    printstats();
+#endif       
+
   	return 0;
 }
