@@ -72,13 +72,13 @@ bool isBoxIdent(Tree t0, const char** str)
 *****************************************************************************/
 
 Tree boxInt(int n) 					{ return tree(n); 	}
-Tree boxReal(float n) 				{ return tree(n); 	}
+Tree boxReal(double n) 				{ return tree(n); 	}
 
 bool isBoxInt(Tree t)				{ return isInt(t->node());	}
-bool isBoxReal(Tree t)				{ return isFloat(t->node()); }
+bool isBoxReal(Tree t)				{ return isDouble(t->node()); }
 
 bool isBoxInt(Tree t, int* i)		{ return isInt(t->node(), i);	}
-bool isBoxReal(Tree t, float* r)	{ return isFloat(t->node(), r); }
+bool isBoxReal(Tree t, double* r)	{ return isDouble(t->node(), r); }
 
 
 /*****************************************************************************
@@ -235,16 +235,38 @@ Tree boxWithLocalDef (Tree body, Tree ldef)					{ return tree(BOXWITHLOCALDEF, b
 bool isBoxWithLocalDef (Tree t, Tree& body, Tree& ldef)		{ return isTree(t, BOXWITHLOCALDEF, body, ldef); }
 
 
+/*****************************************************************************
+                        Boxes modif local definitions
+*****************************************************************************/
+Sym BOXMODIFLOCALDEF 	= symbol ("BoxModifLocalDef");
+
+
+Tree boxModifLocalDef (Tree body, Tree ldef)                { return tree(BOXMODIFLOCALDEF, body, ldef); }
+bool isBoxModifLocalDef (Tree t, Tree& body, Tree& ldef)	{ return isTree(t, BOXMODIFLOCALDEF, body, ldef); }
+
+
 /*****************************************************************************
 							 Modules
 *****************************************************************************/
-Sym BOXCOMPONENT 	= symbol ("BoxComponent");
 
-Tree boxComponent (Tree filename)							{ return tree(BOXCOMPONENT, filename); }
-bool isBoxComponent	(Tree s, Tree& filename)				{ return isTree(s, BOXCOMPONENT, filename); }
+Sym BOXENVIRONMENT  = symbol ("BoxEnvironment");
+
+Tree boxEnvironment ()                                      { return tree(BOXENVIRONMENT); }
+bool isBoxEnvironment (Tree s)                              { return isTree(s, BOXENVIRONMENT); }
+
+Sym BOXCOMPONENT    = symbol ("BoxComponent");
+
+Tree boxComponent (Tree filename)                           { return tree(BOXCOMPONENT, filename); }
+bool isBoxComponent (Tree s, Tree& filename)                { return isTree(s, BOXCOMPONENT, filename); }
+
+Sym BOXLIBRARY      = symbol ("BoxLibrary");
+
+Tree boxLibrary (Tree filename)                             { return tree(BOXLIBRARY, filename); }
+bool isBoxLibrary (Tree s, Tree& filename)                  { return isTree(s, BOXLIBRARY, filename); }
 
 
-Sym IMPORTFILE		= symbol ("ImportFile");
+Sym IMPORTFILE      = symbol ("ImportFile");
+
 Tree importFile(Tree filename)								{ return tree(IMPORTFILE, filename); }
 bool isImportFile(Tree s, Tree& filename)					{ return isTree(s, IMPORTFILE, filename); }
 
