@@ -113,7 +113,7 @@ static void printRule(ostream& fout, Tree rule)
 ostream& boxpp::print (ostream& fout) const
 {
 	int		i, id;
-	float	r;
+	double	r;
 	prim0	p0;
 	prim1	p1;
 	prim2	p2;
@@ -234,12 +234,15 @@ ostream& boxpp::print (ostream& fout) const
 		fout << ')';
 
 	}
-	else if (isClosure(box, abstr, genv, vis, lenv)) {
-		fout << "closure[" << boxpp(abstr)
-			<< ", genv = " << envpp(genv)
-			<< ", lenv = " << envpp(lenv)
-			<< "]";
-	}
+    else if (isBoxEnvironment(box)) {
+        fout << "environment";
+    }
+    else if (isClosure(box, abstr, genv, vis, lenv)) {
+        fout << "closure[" << boxpp(abstr)
+            << ", genv = " << envpp(genv)
+            << ", lenv = " << envpp(lenv)
+            << "]";
+    }
 	else if (isBoxComponent(box, label)) {
 		fout << "component("
 			<< tree2str(label) << ')';
