@@ -50,13 +50,14 @@
 #include "devLib.h"
 #include "ppbox.hh"
 #include "xtended.hh"
-#include "eval.hh"
 #include "occurrences.hh"
 #include "boxcomplexity.h"
 
 #include "schema.h"
 #include "drawschema.hh"
 #include "compatibility.hh"
+#include "names.hh"
+
 
 #if 0
 #define linkcolor "#b3d1dc"
@@ -209,7 +210,7 @@ static bool pendingDrawing(Tree& t)
 
 /**
  * Write a top level diagram. A top level diagram
- * is decorated with is definition name property
+ * is decorated with its definition name property
  * and is drawn in an individual file
  */
 static void writeSchemaFile(Tree bd)
@@ -221,7 +222,13 @@ static void writeSchemaFile(Tree bd)
 
 	gOccurrences = new Occurrences(bd);
 
-	bool hasname = getDefNameProperty(bd, id); assert(hasname);
+	bool hasname = getDefNameProperty(bd, id); 
+
+	//assert(hasname);
+	if (!hasname) {
+		// create an arbitrary name 
+		id = tree(Node(unique("diagram_")));
+	}
 
 	// generate legal file name for the schema
 	stringstream s1; s1 << legalFileName(bd, 1024, temp) << "." << gDevSuffix;
@@ -358,7 +365,7 @@ static schema* generateInsideSchema(Tree t)
 {
 	Tree a, b, ff, l, type,name,file;
 	int		i;
-	float	r;
+	double	r;
 	prim0	p0;
 	prim1	p1;
 	prim2	p2;
