@@ -39,13 +39,17 @@
 #include "signals.hh"
 #include "xtended.hh"
 #include "loopDetector.hh"
+#include "property.hh"
+#include "names.hh"
+#include "compatibility.hh"
+
 
 #include <assert.h>
 extern SourceReader	gReader;
-extern int gMaxNameSize;
+extern int  gMaxNameSize;
 extern bool gPatternEvalMode;
 extern bool	gSimpleNames;
-
+extern bool gSimplifyDiagrams;
 // History
 // 23/05/2005 : New environment management
 
@@ -62,10 +66,9 @@ static Tree 	iterateSum (Tree id, int num, Tree body, Tree visited, Tree localVa
 static Tree 	iterateProd (Tree id, int num, Tree body, Tree visited, Tree localValEnv);
 static Tree 	larg2par (Tree larg);
 static int 		eval2int (Tree exp, Tree visited, Tree localValEnv);
-static float	eval2float (Tree exp, Tree visited, Tree localValEnv);
+static double   eval2double (Tree exp, Tree visited, Tree localValEnv);
 static const char * evalLabel (const char* l, Tree visited, Tree localValEnv);
 
-static Tree 	pushMultiClosureDefs(Tree ldefs, Tree visited, Tree lenv);
 static Tree		evalIdDef(Tree id, Tree visited, Tree env);
 
 
@@ -102,6 +105,15 @@ Tree evalprocess (Tree eqlist)
 }
 
 
+/* Eval a documentation expression. */
+
+Tree evaldocexpr (Tree docexpr, Tree eqlist)
+{
+	return a2sb(eval(docexpr, nil, pushMultiClosureDefs(eqlist, nil, nil)));
+}
+
+
+
 // Private Implementation
 //------------------------
 
@@ -112,15 +124,24 @@ Tree evalprocess (Tree eqlist)
  * @return an expression where abstractions have been replaced by symbolic boxes
  */
 
+property<Tree> gSymbolicBoxProperty;
+
 static Tree real_a2sb(Tree exp);
 
 static Tree a2sb(Tree exp)
 {
-	Tree	id;
-	Tree 	result = real_a2sb(exp);
+    Tree    result;
+    Tree    id;
+
+    if (gSymbolicBoxProperty.get(exp, result)) {
+        return result;
+    }
+
+	result = real_a2sb(exp);
 	if (result != exp && getDefNameProperty(exp, id)) {
 		setDefNameProperty(result, id);		// propagate definition name property when needed
 	}
+    gSymbolicBoxProperty.set(exp, result);
 	return result;
 }
 
@@ -154,6 +175,9 @@ static Tree real_a2sb(Tree exp)
 			// propagate definition name property when needed
 			if (getDefNameProperty(exp, name)) setDefNameProperty(result, name);
 			return result;
+
+        } else if (isBoxEnvironment(abstr)) {
+            return abstr;
 	
 		} else {
 			evalerror(yyfilename, -1, " a2sb : internal error : not an abstraction inside closure ", exp);
@@ -177,59 +201,27 @@ static Tree real_a2sb(Tree exp)
 
 	} else {
 		// it is a constructor : transform each branches
-		Tree B[4];
-		for (int i = 0; i < exp->arity(); i++) {
-			B[i] = a2sb(exp->branch(i));
+        unsigned int    ar = exp->arity();
+		tvec            B(ar);
+        bool            modified = false;
+		for (unsigned int i = 0; i < ar; i++) {
+            Tree b = exp->branch(i);
+            Tree m = a2sb(b);
+            B[i] = m;
+            if (b != m) modified=true;
 		}
-		return replaceBoxNumeric(CTree::make(exp->node(), exp->arity(), B));
+        Tree r = (modified) ? CTree::make(exp->node(), B) : exp;
+        if (gSimplifyDiagrams) {
+            return replaceBoxNumeric(r);
+        } else {
+            return r;
+        }
+/*
+        if (modified) {
+		return replaceBoxNumeric(CTree::make(exp->node(), exp->arity(), B));*/
 	}
 }
 
-/**
- * Definition name property : a property to keep track of the definition name
- * of an expression. Whenever an identifier is evaluated, it is attached as a
- * property of its definitionObviously there is no perfect solution since a same
- * definition quand be given to different names.
- */
-Tree DEFNAMEPROPERTY = tree(symbol("DEFNAMEPROPERTY"));
-
-void setDefNameProperty(Tree t, Tree id)
-{
-	//cerr << "setDefNameProperty : " << *id << " FOR " << t << "#" << boxpp(t) << endl;
-	setProperty(t, DEFNAMEPROPERTY, id);
-}
-
-void setDefNameProperty(Tree t, const string& name)
-{
-	//cerr << "setDefNameProperty : " << name << " FOR " << t << "#" << boxpp(t) << endl;
-	int		n = name.size();
-	int 	m = (gMaxNameSize>1023) ? 1023 : gMaxNameSize;
-	if (n > m) {
-		// the name is too long we reduce it to 2/3 of maxsize
-		char 	buf[1024];
-		int i = 0;
-		// copy first third
-		for (; i < m/3; i++) { buf[i] = name[i]; }
-		// add ...
-		buf[i++] = '.';
-		buf[i++] = '.';
-		buf[i++] = '.';
-		// copy last third
-		for (int c = n-m/3; c<n; c++, i++) { buf[i] = name[c]; }
-		buf[i] = 0;
-		setProperty(t, DEFNAMEPROPERTY, tree(buf));
-	} else {
-		setProperty(t, DEFNAMEPROPERTY, tree(name.c_str()));
-	}
-
-}
-
-bool getDefNameProperty(Tree t, Tree& id)
-{
-	//cerr << "getDefNameProperty of : " << t << endl;
-	return getProperty(t, DEFNAMEPROPERTY, id);
-}
-
 static bool autoName(Tree exp , Tree& id)
 {
 	stringstream s; s << boxpp(exp);
@@ -365,23 +357,49 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 	// Modules
 	//--------
 
-	} else if (isBoxAccess(exp, body, var)) {
-		Tree val = eval(body, visited, localValEnv);
-		if (isClosure(val, exp2, notused, visited2, lenv2)) {
-			// it is a closure, we have an environment to access
-			return eval(closure(var,notused,visited2,lenv2), visited, localValEnv);
-		} else {
-			evalerror(getDefFileProp(exp), getDefLineProp(exp), "No environment to access ", exp);
-			exit(1);
-		}
+    } else if (isBoxAccess(exp, body, var)) {
+        Tree val = eval(body, visited, localValEnv);
+        if (isClosure(val, exp2, notused, visited2, lenv2)) {
+            // it is a closure, we have an environment to access
+            return eval(closure(var,notused,visited2,lenv2), visited, localValEnv);
+        } else {
+            evalerror(getDefFileProp(exp), getDefLineProp(exp), "No environment to access ", exp);
+            exit(1);
+        }
 
-	} else if (isBoxComponent(exp, label)) {
-		string 	fname 	= tree2str(label);
-		Tree 	eqlst 	= gReader.expandlist(gReader.getlist(fname));
-		Tree	res 	= closure(boxIdent("process"), nil, nil, pushMultiClosureDefs(eqlst, nil, nil));
-		setDefNameProperty(res, label);
-		//cerr << "component is " << boxpp(res) << endl;
-		return res;
+//////////////////////en chantier////////////////////////////
+
+    } else if (isBoxModifLocalDef(exp, body, ldef)) {
+        Tree val = eval(body, visited, localValEnv);
+        if (isClosure(val, exp2, notused, visited2, lenv2)) {
+            // we rebuild the closure using a copy of the original environment
+            // modified with some new definitions
+            Tree lenv3 = copyEnvReplaceDefs(lenv2, ldef, visited2, localValEnv);
+            return eval(closure(exp2,notused,visited2,lenv3), visited, localValEnv);
+        } else {
+
+            evalerror(getDefFileProp(exp), getDefLineProp(exp), "not a closure ", val);
+            evalerror(getDefFileProp(exp), getDefLineProp(exp), "No environment to access ", exp);
+            exit(1);
+        }
+
+///////////////////////////////////////////////////////////////////
+
+    } else if (isBoxComponent(exp, label)) {
+        string  fname   = tree2str(label);
+        Tree    eqlst   = gReader.expandlist(gReader.getlist(fname));
+        Tree    res     = closure(boxIdent("process"), nil, nil, pushMultiClosureDefs(eqlst, nil, nil));
+        setDefNameProperty(res, label);
+        //cerr << "component is " << boxpp(res) << endl;
+        return res;
+
+    } else if (isBoxLibrary(exp, label)) {
+        string  fname   = tree2str(label);
+        Tree    eqlst   = gReader.expandlist(gReader.getlist(fname));
+        Tree    res     = closure(boxEnvironment(), nil, nil, pushMultiClosureDefs(eqlst, nil, nil));
+        setDefNameProperty(res, label);
+        //cerr << "component is " << boxpp(res) << endl;
+        return res;
 
 
 	// user interface elements
@@ -403,28 +421,28 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 		const char* l1 = tree2str(label);
 		const char* l2= evalLabel(l1, visited, localValEnv);
 		return ( boxVSlider(tree(l2),
-					tree(eval2float(cur, visited, localValEnv)),
-					tree(eval2float(lo, visited, localValEnv)),
-					tree(eval2float(hi, visited, localValEnv)),
-					tree(eval2float(step, visited, localValEnv))));
+					tree(eval2double(cur, visited, localValEnv)),
+					tree(eval2double(lo, visited, localValEnv)),
+					tree(eval2double(hi, visited, localValEnv)),
+					tree(eval2double(step, visited, localValEnv))));
 
 	} else if (isBoxHSlider(exp, label, cur, lo, hi, step)) {
 		const char* l1 = tree2str(label);
 		const char* l2= evalLabel(l1, visited, localValEnv);
 		return ( boxHSlider(tree(l2),
-					tree(eval2float(cur, visited, localValEnv)),
-					tree(eval2float(lo, visited, localValEnv)),
-					tree(eval2float(hi, visited, localValEnv)),
-					tree(eval2float(step, visited, localValEnv))));
+					tree(eval2double(cur, visited, localValEnv)),
+					tree(eval2double(lo, visited, localValEnv)),
+					tree(eval2double(hi, visited, localValEnv)),
+					tree(eval2double(step, visited, localValEnv))));
 
 	} else if (isBoxNumEntry(exp, label, cur, lo, hi, step)) {
 		const char* l1 = tree2str(label);
 		const char* l2= evalLabel(l1, visited, localValEnv);
 		return (boxNumEntry(tree(l2),
-					tree(eval2float(cur, visited, localValEnv)),
-					tree(eval2float(lo, visited, localValEnv)),
-					tree(eval2float(hi, visited, localValEnv)),
-					tree(eval2float(step, visited, localValEnv))));
+					tree(eval2double(cur, visited, localValEnv)),
+					tree(eval2double(lo, visited, localValEnv)),
+					tree(eval2double(hi, visited, localValEnv)),
+					tree(eval2double(step, visited, localValEnv))));
 
 	} else if (isBoxVGroup(exp, label, arg)) {
 		const char* l1 = tree2str(label);
@@ -445,15 +463,15 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 		const char* l1 = tree2str(label);
 		const char* l2= evalLabel(l1, visited, localValEnv);
 		return boxHBargraph(tree(l2),
-					tree(eval2float(lo, visited, localValEnv)),
-					tree(eval2float(hi, visited, localValEnv)));
+					tree(eval2double(lo, visited, localValEnv)),
+					tree(eval2double(hi, visited, localValEnv)));
 
 	} else if (isBoxVBargraph(exp, label, lo, hi)) {
 		const char* l1 = tree2str(label);
 		const char* l2= evalLabel(l1, visited, localValEnv);
 		return boxVBargraph(tree(l2),
-					tree(eval2float(lo, visited, localValEnv)),
-					tree(eval2float(hi, visited, localValEnv)));
+					tree(eval2double(lo, visited, localValEnv)),
+					tree(eval2double(hi, visited, localValEnv)));
 
 	// lambda calculus
 	//----------------
@@ -468,16 +486,23 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 		return applyList(	eval(fun, visited, localValEnv),
 							revEvalList(arg, visited, localValEnv) );
 
-	} else if (isBoxAbstr(exp)) {
-		// it is an abstraction : return a closure
-		return closure(exp, nil, visited, localValEnv);
+    } else if (isBoxAbstr(exp)) {
+        // it is an abstraction : return a closure
+        return closure(exp, nil, visited, localValEnv);
+
+    } else if (isBoxEnvironment(exp)) {
+        // environment : return also a closure
+        return closure(exp, nil, visited, localValEnv);
 
 	} else if (isClosure(exp, exp2, notused, visited2, lenv2)) {
 
-		if (isBoxAbstr(exp2)) {
-			// a 'real' closure
-			return closure(exp2, nil, setUnion(visited,visited2), lenv2);
-		} else {
+        if (isBoxAbstr(exp2)) {
+            // a 'real' closure
+            return closure(exp2, nil, setUnion(visited,visited2), lenv2);
+        } else if (isBoxEnvironment(exp2)) {
+            // a 'real' closure
+            return closure(exp2, nil, setUnion(visited,visited2), lenv2);
+        } else {
 			// it was a suspended evaluation
 			return eval(exp2, setUnion(visited,visited2), lenv2);
 		}
@@ -523,8 +548,8 @@ static Tree realeval (Tree exp, Tree visited, Tree localValEnv)
 		return exp;
 
 	} else {
-		cout << "ERROR : EVAL don't intercept : " << *exp << endl;
-		exit(1);
+		cerr << "ERROR : EVAL don't intercept : " << *exp << endl;
+		assert(false);
 	}
 }
 
@@ -562,7 +587,7 @@ bool getNumericProperty(Tree t, Tree& num)
 static Tree replaceBoxNumeric (Tree exp)
 {
 	int 	numInputs, numOutputs;
-	float 	x; 
+	double 	x; 
 	int		i;
 	Tree	out;
 	
@@ -611,7 +636,7 @@ Tree simplifyPattern (Tree value)
 static bool isBoxNumeric (Tree in, Tree& out)
 {
 	int 	numInputs, numOutputs;
-	float 	x; 
+	double 	x; 
 	int		i;
 	Tree 	v;
 	
@@ -655,19 +680,19 @@ static Tree patternSimplification (Tree pattern)
 
 
 /**
- * Eval a block diagram to a float.
+ * Eval a block diagram to a double.
  *
- * Eval a block diagram that represent a float constant. This function first eval
+ * Eval a block diagram that represent a double constant. This function first eval
  * a block diagram to its normal form, then check it represent a numerical value (a
  * block diagram of type : 0->1) then do a symbolic propagation and try to convert the
- * resulting signal to a float.
+ * resulting signal to a double.
  * @param exp the expression to evaluate
  * @param globalDefEnv the global environment
  * @param visited list of visited definition to detect recursive definitions
  * @param localValEnv the local environment
  * @return a block diagram in normal form
  */
-static float eval2float (Tree exp, Tree visited, Tree localValEnv)
+static double eval2double (Tree exp, Tree visited, Tree localValEnv)
 {
 	Tree diagram = eval(exp, visited, localValEnv);
 	int numInputs, numOutputs;
@@ -901,6 +926,58 @@ static Tree iterateProd (Tree id, int num, Tree body, Tree visited, Tree localVa
 	return res;
 }
 
+/**
+ * Compute the sum of outputs of a list of boxes. The sum is
+ * valid if all the boxes have a valid boxType
+ *
+ * @param boxlist the list of boxes
+ * @param outputs sum of outputs of the boxes
+ * @return true if outputs is valid, false otherwise
+ */
+ #if 1
+static bool boxlistOutputs(Tree boxlist, int* outputs)
+{
+    int ins, outs;
+
+    *outputs = 0;
+    while (!isNil(boxlist))
+    {
+    	if (getBoxType(hd(boxlist), &ins, &outs)) {
+            *outputs += outs;
+      	} else {
+      		// arbitrary output arity set to 1
+      		// when can't be determined
+      		*outputs += 1;
+      	}
+        boxlist = tl(boxlist);
+    }
+    return isNil(boxlist);
+}
+#else
+static bool boxlistOutputs(Tree boxlist, int* outputs)
+{
+    int ins, outs;
+
+    *outputs = 0;
+    while (!isNil(boxlist) && getBoxType(hd(boxlist), &ins, &outs)) {
+            *outputs += outs;
+            boxlist = tl(boxlist);
+    }
+    return isNil(boxlist);
+}
+#endif
+
+/**
+ * repeat n times a wire
+ */
+static Tree nwires(int n)
+{
+	Tree l = nil;
+	while (n--) { l = cons(boxWire(), l); }
+	return l;
+}
+
+
 /**
  * Apply a function to a list of arguments. 
  * Apply a function F to a list of arguments (a,b,c,...).
@@ -960,22 +1037,61 @@ static Tree applyList (Tree fun, Tree larg)
 				//return eval(body, nil, localValEnv);
 				return applyList(eval(body, nil, localValEnv), tl(larg));
 			} else {
-				cout << "wrong result from pattern matching (not a closure) : " << boxpp(result) << endl;
+				cerr << "wrong result from pattern matching (not a closure) : " << boxpp(result) << endl;
 				return boxError();
 			}
 		}			
 	}
 	if (!isClosure(fun, abstr, globalDefEnv, visited, localValEnv)) {
-		if (isNil(tl(larg)) && isBoxPrim2(fun, &p2) && (p2 != sigPrefix)) {
-			return boxSeq(boxPar(boxWire(), hd(larg)), fun);
+		// principle : f(a,b,c,...) ==> (a,b,c,...):f
+         int ins, outs;
+         
+         // check arity of function
+         if (!getBoxType(fun, &ins, &outs)) {
+         	// we can't determine the input arity of the expression
+         	// hope for the best
+         	return boxSeq(larg2par(larg), fun);
+         }
+ 
+         // check arity of arg list
+         if (!boxlistOutputs(larg,&outs)) {
+         	// we don't know yet the output arity of larg. Therefore we can't
+         	// do any arity checking nor add _ to reach the required number of arguments
+            // cerr << "warning : can't infere the type of : " << boxpp(larg) << endl;
+         	return boxSeq(larg2par(larg), fun);
+         }
+		
+		if (outs > ins) {
+			cerr << "too much arguments : " << outs << ", instead of : " << ins << endl;
+            cerr << "when applying : " << boxpp(fun) << endl
+                 << "           to : " << boxpp(larg) << endl;
+			assert(false);
 		}
-		return boxSeq(larg2par(larg), fun);
+		
+        if (    (outs == 1)
+            &&
+                (  ( isBoxPrim2(fun, &p2) && (p2 != sigPrefix) )
+                || ( getUserData(fun) && ((xtended*)getUserData(fun))->isSpecialInfix() ) ) ) {
+            // special case : /(3) ==> _,3 : /
+            Tree larg2 = concat(nwires(ins-outs), larg);
+            return boxSeq(larg2par(larg2), fun);
+
+        } else {
+
+			Tree larg2 = concat(larg, nwires(ins-outs));
+            return boxSeq(larg2par(larg2), fun);
+        }
 	}
 
-	if (!isBoxAbstr(abstr, id, body)) {
-		evalerror(yyfilename, -1, "(internal) not an abstraction inside closure", fun);
-		exit(1);
-	}
+    if (isBoxEnvironment(abstr)) {
+        evalerrorbox(yyfilename, -1, "an environment can't be used as a function", fun);
+        exit(1);
+    }
+
+    if (!isBoxAbstr(abstr, id, body)) {
+        evalerror(yyfilename, -1, "(internal) not an abstraction inside closure", fun);
+        exit(1);
+    }
 
 	// try to synthetise a  name from the function name and the argument name
 	{
@@ -1037,113 +1153,6 @@ static Tree larg2par (Tree larg)
 
 
 
-//-----------------------new environment management----------------------------
-//
-// The environement is made of layers. Each layer contains a set of definitions
-// stored as properties of the layer. Each definition can refers to other
-// definitions of the same layer or of subsequent layers. Recursive
-// definitions are not allowed. Multiple defintions of the same symbol
-// in a layer is allowed but generate a warning when the definition is
-// different
-//-----------------------------------------------------------------------------
-
-
-
-/**
- * Push a new (unique) empty layer (where multiple definitions can be stored)
- * on top of an existing environment.
- * @param lenv the old environment
- * @return the new environment
-*/
-static Tree pushNewLayer(Tree lenv)
-{
-	return tree(unique("ENV_LAYER"), lenv);
-}
-
-
-/**
- * Add a definition (as a property) to the current top level layer. Check
- * and warn for multiple definitions.
- * @param id the symbol id to be defined
- * @param def the definition to be binded to the symbol id
- * @param lenv the environment where to add this new definition
-*/
-static void addLayerDef(Tree id, Tree def, Tree lenv)
-{
-	// check for multiple definitions of a symbol in the same layer
-	Tree olddef;
-	if (getProperty(lenv, id, olddef)) {
-		if (def == olddef) {
-			evalwarning(getDefFileProp(id), getDefLineProp(id), "equivalent re-definitions of", id);
-		} else {
-            fprintf(stderr, "%s:%d: ERROR: redefinition of symbols are not allowed : ", getDefFileProp(id), getDefLineProp(id)); 
-            print(id,stderr); 
-            fprintf(stderr, " is already defined in file \"%s\" line %d \n", getDefFileProp(id), getDefLineProp(id)); 
-            gErrorCount++;
-		}
-	}
-	setProperty(lenv, id, def);
-}
-
-
-/**
- * Push a new layer and add a single definition.
- * @param id the symbol id to be defined
- * @param def the definition to be binded to the symbol id
- * @param lenv the environment where to push the layer and add the definition
- * @return the new environment
- */
-Tree pushValueDef(Tree id, Tree def, Tree lenv)
-{
-	Tree lenv2 = pushNewLayer(lenv);
-	addLayerDef(id, def, lenv2);
-	return lenv2;
-}
-
-
-/**
- * Push a new layer with multiple definitions creating the appropriate closures
- * @param ldefs list of pairs (symbol id x definition) to be binded to the symbol id
- * @param visited set of visited symbols (used for recursive definition detection)
- * @param lenv the environment where to push the layer and add all the definitions
- * @return the new environment
-*/
-static Tree pushMultiClosureDefs(Tree ldefs, Tree visited, Tree lenv)
-{
-	Tree lenv2 = pushNewLayer(lenv);
-	while (!isNil(ldefs)) {
-		Tree def = hd(ldefs);
-		Tree id = hd(def);
-		Tree rhs= tl(def);
-		Tree cl = closure(tl(def),nil,visited,lenv2);
-		stringstream s; s << boxpp(id);
-		if (!isBoxCase(rhs)) setDefNameProperty(cl,s.str());
-		addLayerDef( id, cl, lenv2 );
-		ldefs = tl(ldefs);
-	}
-	return lenv2;
-}
-
-
-/**
- * Search the environment for the definition of a symbol
- * ID and return it. 
- * @param id the symbol ID to search
- * @param def where to store the definition if any
- * @param lenv the environment
- * @return true if a definition was found
- */
-bool searchIdDef(Tree id, Tree& def, Tree lenv)
-{
-	// search the environment until a definition is found
-	// or nil (the empty environment) is reached
-	while (!isNil(lenv) && !getProperty(lenv, id, def)) {
-		lenv = lenv->branch(0);
-	}
-	return !isNil(lenv);
-}
-
-
 
 /**
  * Search the environment for the definition of a symbol
