@@ -2,6 +2,8 @@
 #include "Text.hh"
 #include <math.h>
 
+#include "floats.hh"
+
 class CosPrim : public xtended
 {
 
@@ -29,7 +31,7 @@ class CosPrim : public xtended
 	virtual Tree	computeSigOutput (const vector<Tree>& args) {
 		num n;
 		if (isNum(args[0],n)) {
-			return tree(cosf(float(n)));
+			return tree(cos(double(n)));
 		} else {
 			return tree(symbol(), args[0]);
 		}
@@ -37,7 +39,18 @@ class CosPrim : public xtended
 		
 	virtual string 	generateCode (Klass* klass, const vector<string>& args, const vector<Type>& types)
 	{
-		return subst("cosf($0)", args[0]);
+		assert (args.size() == arity());
+		assert (types.size() == arity());
+		
+        return subst("cos$1($0)", args[0], isuffix());
+	}
+	
+	virtual string 	generateLateq (Lateq* lateq, const vector<string>& args, const vector<Type>& types)
+	{
+		assert (args.size() == arity());
+		assert (types.size() == arity());
+		
+        return subst("\\cos\\left($0\\right)", args[0]);
 	}
 	
 };
