@@ -30,8 +30,8 @@ BinOp* gBinOpTable[] = {
 	new BinOp("+","add_vec","add_scal", &addNode, &isZero, &isZero, 6),
 	new BinOp("-","sub_vec","sub_scal", &subNode, &noNtrl, &isZero, 6),
 	new BinOp("*","mul_vec","mul_scal", &mulNode, &isOne, &isOne, 7),
-	new BinOp("/","div_vec","div_scal", &divNode, &noNtrl, &isOne, 7),
-	new BinOp("%","mod_vec","mod_scal", &remNode, &noNtrl, &noNtrl, 7),
+	new BinOp("/","div_vec","div_scal", &divNode, &noNtrl, &isOne, 8),
+	new BinOp("%","mod_vec","mod_scal", &remNode, &noNtrl, &noNtrl, 8),
 
 	new BinOp("<<","shift_left_vec","shift_left_scal", &lshNode, &noNtrl, &isZero, 7),
 	new BinOp(">>","shift_right_vec","shift_right_scal", &rshNode, &noNtrl, &isZero, 7),
@@ -48,3 +48,26 @@ BinOp* gBinOpTable[] = {
 	new BinOp("^","xor_vec","xor_scal", &xorNode, &noNtrl, &noNtrl, 7)
 };
 
+
+BinOp* gBinOpLateqTable[] = { 
+
+	new BinOp("+","add_vec","add_scal", &addNode, &isZero, &isZero, 6),
+	new BinOp("-","sub_vec","sub_scal", &subNode, &noNtrl, &isZero, 6),
+	new BinOp("*","mul_vec","mul_scal", &mulNode, &isOne, &isOne, 7), // \DeclareMathSymbol{*}{\mathbin}{symbols}{"01}
+	new BinOp("/","div_vec","div_scal", &divNode, &noNtrl, &isOne, 9), // \frac{}{} used in generateBinOp
+	new BinOp("\\bmod","mod_vec","mod_scal", &remNode, &noNtrl, &noNtrl, 8),
+	
+	new BinOp("\\hiderel{\\ll}","shift_left_vec","shift_left_scal", &lshNode, &noNtrl, &isZero, 7),
+	new BinOp("\\hiderel{\\gg}","shift_right_vec","shift_right_scal", &rshNode, &noNtrl, &isZero, 7),
+	
+	new BinOp("\\hiderel{>}","gt_vec","gt_scal", &gtNode, &noNtrl, &noNtrl, 5),
+	new BinOp("\\hiderel{<}","lt_vec","lt_scal", &ltNode, &noNtrl, &noNtrl, 5),
+	new BinOp("\\hiderel{\\geq}","ge_vec","ge_scal", &geNode, &noNtrl, &noNtrl, 5),
+	new BinOp("\\hiderel{\\leq}","le_vec","le_scal", &leNode, &noNtrl, &noNtrl, 5),
+	new BinOp("\\hiderel{\\equiv}","eq_vec","eq_scal", &eqNode, &noNtrl, &noNtrl, 5),
+	new BinOp("\\hiderel{\\neq}","neq_vec","neq_scal", &neNode, &noNtrl, &noNtrl, 5),
+	
+	new BinOp("\\wedge","and_vec","and_scal", &andNode, &isMinusOne, &isMinusOne, 7),
+	new BinOp("\\vee","or_vec","or_scal", &orNode, &isZero, &isZero, 6),
+	new BinOp("\\veebar","xor_vec","xor_scal", &xorNode, &noNtrl, &noNtrl, 7)
+};
