@@ -52,7 +52,12 @@ ostream& ppsig::printfun (ostream& fout, const string& funame, Tree x, Tree y, T
 
 ostream& ppsig::printfun (ostream& fout, const string& funame, Tree x, Tree y, Tree z, Tree zz) const
 {
-	return fout << funame << '(' << ppsig(x,fEnv) << ',' << ppsig(y,fEnv) << ',' << ppsig(z,fEnv) << ',' << ppsig(zz,fEnv) << ')';
+    return fout << funame << '(' << ppsig(x,fEnv) << ',' << ppsig(y,fEnv) << ',' << ppsig(z,fEnv) << ',' << ppsig(zz,fEnv) << ')';
+}
+
+ostream& ppsig::printfun (ostream& fout, const string& funame, Tree x, Tree y, Tree z, Tree z2, Tree z3) const
+{
+    return fout << funame << '(' << ppsig(x,fEnv) << ',' << ppsig(y,fEnv) << ',' << ppsig(z,fEnv) << ',' << ppsig(z2,fEnv) << ',' << ppsig(z3,fEnv) << ')';
 }
 
 ostream& ppsig::printui (ostream& fout, const string& funame, Tree label) const
@@ -149,6 +154,12 @@ ostream& ppsig::printrec (ostream& fout, Tree var, Tree lexp, bool hide) const
 	return fout;
 }
 
+ostream& ppsig::printrec (ostream& fout, Tree lexp, bool hide) const
+{
+	fout << "debruijn(" << ppsig(lexp,fEnv) << ")";
+	return fout;
+}
+
 ostream& ppsig::printextended (ostream& fout, Tree sig) const
 {
 	string 		sep = "";
@@ -166,36 +177,28 @@ ostream& ppsig::printextended (ostream& fout, Tree sig) const
 	
 ostream& ppsig::print (ostream& fout) const
 {
-#if 0
 	int 	i;
-	float	r;
-	Tree 	c, sel, x, y, z, var, le, label, id, ff, largs, type, name, file;
-	//cerr << "ppsig::print of " << *sig <<endl;
-	 if ( isSigBinOp(sig, &i, x, y) )			{ printinfix(fout, gBinOpTable[i]->fName, gBinOpTable[i]->fPriority, x, y);  }
-	 else { cerr << "ppsig::print of " << *sig <<endl; }
-#endif
- 
-
-
-#if 1
-	int 	i;
-	float	r;
-	Tree 	c, sel, x, y, z, var, le, label, id, ff, largs, type, name, file;
+	double	r;
+    Tree 	c, sel, x, y, z, u, var, le, label, id, ff, largs, type, name, file;
 
 		  if ( isList(sig) ) 						{ printlist(fout, sig); }
-	else if ( isProj(sig, &i, x) ) 				{ fout << "proj" << i << '(' << ppsig(x, fEnv) << ')';	}
-	else if ( isRec(sig, var, le) )				{ printrec(fout, var, le, fHideRecursion && (getRecursivness(sig)==0) ); }
+	else if ( isProj(sig, &i, x) ) 					{ fout << "proj" << i << '(' << ppsig(x, fEnv) << ')';	}
+	else if ( isRec(sig, var, le) )					{ printrec(fout, var, le, fHideRecursion /*&& (getRecursivness(sig)==0)*/ ); }
+
+	// debruinj notation
+	else if ( isRec(sig, le) )						{ printrec(fout, le, fHideRecursion ); }
+	else if ( isRef(sig, i) )						{ fout << "REF[" << i << "]"; }
 	
 	else if ( getUserData(sig) ) 					{ printextended(fout, sig); }
 	else if ( isSigInt(sig, &i) ) 					{ fout << i; }
-	else if ( isSigReal(sig, &r) ) 				{ fout << r; }
-	else if ( isSigInput(sig, &i) ) 				{ fout << "IN" << i; }
+	else if ( isSigReal(sig, &r) ) 					{ fout << r; }
+	else if ( isSigInput(sig, &i) ) 				{ fout << "IN[" << i << "]"; }
 	else if ( isSigOutput(sig, &i, x) ) 			{ printout(fout, i, x) ; }
 	
 	else if ( isSigDelay1(sig, x) ) 				{ fout << ppsig(x, fEnv, 9) << "'"; }
 	//else if ( isSigFixDelay(sig, x, y) ) 			{ printinfix(fout, "@", 8, x, y); 	}
 	else if ( isSigFixDelay(sig, x, y) ) 			{ printFixDelay(fout, x, y); 	}
-	else if ( isSigPrefix(sig, x, y) ) 			{ printfun(fout, "prefix", x, y); }
+	else if ( isSigPrefix(sig, x, y) ) 				{ printfun(fout, "prefix", x, y); }
 	else if ( isSigIota(sig, x) ) 					{ printfun(fout, "iota", x); }
 	else if ( isSigBinOp(sig, &i, x, y) )			{ printinfix(fout, gBinOpTable[i]->fName, gBinOpTable[i]->fPriority, x, y);  }
 	else if ( isSigFFun(sig, ff, largs) )			{ printff(fout, ff, largs); }
@@ -206,12 +209,16 @@ ostream& ppsig::print (ostream& fout) const
 	else if ( isSigWRTbl(sig, id, x, y, z) )		{ printfun(fout, "write", x, y, z);	}
 	else if ( isSigRDTbl(sig, x, y) )				{ printfun(fout, "read", x, y);	}
 	else if ( isSigGen(sig, x) ) 					{ fout << ppsig(x, fEnv, fPriority); }
-	
+
+    else if ( isSigDocConstantTbl(sig, x, y) )      { printfun(fout, "docConstantTbl", x, y);	}
+    else if ( isSigDocWriteTbl(sig, x, y, z, u) )   { printfun(fout, "docWriteTbl", x, y, z, u);	}
+    else if ( isSigDocAccessTbl(sig, x, y) )        { printfun(fout, "docAccessTbl", x, y);	}
+
 	else if ( isSigSelect2(sig, sel, x, y) ) 		{ printfun(fout, "select2", sel, x, y); }
 	else if ( isSigSelect3(sig, sel, x, y, z) ) 	{ printfun(fout, "select3", sel, x, y, z); }
 	
 	else if ( isSigIntCast(sig, x) ) 				{ printfun(fout, "int", x); }
-	else if ( isSigFloatCast(sig, x) ) 			{ printfun(fout, "float", x);  }
+	else if ( isSigFloatCast(sig, x) )				{ printfun(fout, "float", x);  }
 	
 	else if ( isSigButton(sig, label) ) 			{ printui(fout, "button", label); }
 	else if ( isSigCheckbox(sig, label) ) 			{ printui(fout, "checkbox", label);  }
@@ -226,7 +233,6 @@ ostream& ppsig::print (ostream& fout) const
 		cerr << "ERROR, ppsig doesn't recognize signal : " << *sig << endl;
 		exit(1);
 	}
-#endif
 	return fout;
 }
 
