@@ -4,10 +4,16 @@
    then loaded dynamically by Pd as an external. */
 
 #include <stdlib.h>
+#include <string.h>
 #include <math.h>
+#include <string>
 
 using namespace std;
-#include <string>
+
+struct Meta 
+{
+    void declare (const char* key, const char* value) {}
+};
 
 //-------------------------------------------------------------------
 // Generic min and max using c++ inline
@@ -90,28 +96,30 @@ public:
   UI() : fStopped(false) {}
   virtual ~UI() {}
 	
-  virtual void addButton(char* label, float* zone) = 0;
-  virtual void addToggleButton(char* label, float* zone) = 0;
-  virtual void addCheckButton(char* label, float* zone) = 0;
-  virtual void addVerticalSlider(char* label, float* zone, float init, float min, float max, float step) = 0;
-  virtual void addHorizontalSlider(char* label, float* zone, float init, float min, float max, float step) = 0;
-  virtual void addNumEntry(char* label, float* zone, float init, float min, float max, float step) = 0;
+  virtual void addButton(const char* label, float* zone) = 0;
+  virtual void addToggleButton(const char* label, float* zone) = 0;
+  virtual void addCheckButton(const char* label, float* zone) = 0;
+  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step) = 0;
+  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step) = 0;
+  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step) = 0;
 
-  virtual void addNumDisplay(char* label, float* zone, int precision) = 0;
-  virtual void addTextDisplay(char* label, float* zone, char* names[], float min, float max) = 0;
-  virtual void addHorizontalBargraph(char* label, float* zone, float min, float max) = 0;
-  virtual void addVerticalBargraph(char* label, float* zone, float min, float max) = 0;
+  virtual void addNumDisplay(const char* label, float* zone, int precision) = 0;
+  virtual void addTextDisplay(const char* label, float* zone, const char* names[], float min, float max) = 0;
+  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max) = 0;
+  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max) = 0;
 	
-  virtual void openFrameBox(char* label) = 0;
-  virtual void openTabBox(char* label) = 0;
-  virtual void openHorizontalBox(char* label) = 0;
-  virtual void openVerticalBox(char* label) = 0;
+  virtual void openFrameBox(const char* label) = 0;
+  virtual void openTabBox(const char* label) = 0;
+  virtual void openHorizontalBox(const char* label) = 0;
+  virtual void openVerticalBox(const char* label) = 0;
   virtual void closeBox() = 0;
 	
   virtual void run() = 0;
 	
   void stop()	{ fStopped = true; }
   bool stopped() 	{ return fStopped; }
+
+  virtual void declare(float* zone, const char* key, const char* value) {}
 };
 
 /***************************************************************************
@@ -144,30 +152,30 @@ public:
 
 protected:
   string path;
-  void add_elem(ui_elem_type_t type, char *label = NULL);
-  void add_elem(ui_elem_type_t type, char *label, float *zone);
-  void add_elem(ui_elem_type_t type, char *label, float *zone,
+  void add_elem(ui_elem_type_t type, const char *label = NULL);
+  void add_elem(ui_elem_type_t type, const char *label, float *zone);
+  void add_elem(ui_elem_type_t type, const char *label, float *zone,
 		float init, float min, float max, float step);
-  void add_elem(ui_elem_type_t type, char *label, float *zone,
+  void add_elem(ui_elem_type_t type, const char *label, float *zone,
 		float min, float max);
 
 public:
-  virtual void addButton(char* label, float* zone);
-  virtual void addToggleButton(char* label, float* zone);
-  virtual void addCheckButton(char* label, float* zone);
-  virtual void addVerticalSlider(char* label, float* zone, float init, float min, float max, float step);
-  virtual void addHorizontalSlider(char* label, float* zone, float init, float min, float max, float step);
-  virtual void addNumEntry(char* label, float* zone, float init, float min, float max, float step);
+  virtual void addButton(const char* label, float* zone);
+  virtual void addToggleButton(const char* label, float* zone);
+  virtual void addCheckButton(const char* label, float* zone);
+  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
+  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
+  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);
 
-  virtual void addNumDisplay(char* label, float* zone, int precision);
-  virtual void addTextDisplay(char* label, float* zone, char* names[], float min, float max);
-  virtual void addHorizontalBargraph(char* label, float* zone, float min, float max);
-  virtual void addVerticalBargraph(char* label, float* zone, float min, float max);
+  virtual void addNumDisplay(const char* label, float* zone, int precision);
+  virtual void addTextDisplay(const char* label, float* zone, const char* names[], float min, float max);
+  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
+  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
   
-  virtual void openFrameBox(char* label);
-  virtual void openTabBox(char* label);
-  virtual void openHorizontalBox(char* label);
-  virtual void openVerticalBox(char* label);
+  virtual void openFrameBox(const char* label);
+  virtual void openTabBox(const char* label);
+  virtual void openHorizontalBox(const char* label);
+  virtual void openVerticalBox(const char* label);
   virtual void closeBox();
 	
   virtual void run();
@@ -234,7 +242,7 @@ PdUI::~PdUI()
   }
 }
 
-inline void PdUI::add_elem(ui_elem_type_t type, char *label)
+inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
 {
   ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
   if (elems1)
@@ -252,7 +260,7 @@ inline void PdUI::add_elem(ui_elem_type_t type, char *label)
   nelems++;
 }
 
-inline void PdUI::add_elem(ui_elem_type_t type, char *label, float *zone)
+inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
 {
   ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
   if (elems1)
@@ -270,7 +278,7 @@ inline void PdUI::add_elem(ui_elem_type_t type, char *label, float *zone)
   nelems++;
 }
 
-inline void PdUI::add_elem(ui_elem_type_t type, char *label, float *zone,
+inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
 			  float init, float min, float max, float step)
 {
   ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
@@ -289,7 +297,7 @@ inline void PdUI::add_elem(ui_elem_type_t type, char *label, float *zone,
   nelems++;
 }
 
-inline void PdUI::add_elem(ui_elem_type_t type, char *label, float *zone,
+inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
 			  float min, float max)
 {
   ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
@@ -308,43 +316,43 @@ inline void PdUI::add_elem(ui_elem_type_t type, char *label, float *zone,
   nelems++;
 }
 
-void PdUI::addButton(char* label, float* zone)
+void PdUI::addButton(const char* label, float* zone)
 { add_elem(UI_BUTTON, label, zone); }
-void PdUI::addToggleButton(char* label, float* zone)
+void PdUI::addToggleButton(const char* label, float* zone)
 { add_elem(UI_TOGGLE_BUTTON, label, zone); }
-void PdUI::addCheckButton(char* label, float* zone)
+void PdUI::addCheckButton(const char* label, float* zone)
 { add_elem(UI_CHECK_BUTTON, label, zone); }
-void PdUI::addVerticalSlider(char* label, float* zone, float init, float min, float max, float step)
+void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
 { add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
-void PdUI::addHorizontalSlider(char* label, float* zone, float init, float min, float max, float step)
+void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
 { add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
-void PdUI::addNumEntry(char* label, float* zone, float init, float min, float max, float step)
+void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
 { add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }
 
 // FIXME: addNumDisplay and addTextDisplay not implemented in Faust yet?
-void PdUI::addNumDisplay(char* label, float* zone, int precision) {}
-void PdUI::addTextDisplay(char* label, float* zone, char* names[], float min, float max) {}
-void PdUI::addHorizontalBargraph(char* label, float* zone, float min, float max)
+void PdUI::addNumDisplay(const char* label, float* zone, int precision) {}
+void PdUI::addTextDisplay(const char* label, float* zone, const char* names[], float min, float max) {}
+void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
 { add_elem(UI_H_BARGRAPH, label, zone, min, max); }
-void PdUI::addVerticalBargraph(char* label, float* zone, float min, float max)
+void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
 { add_elem(UI_V_BARGRAPH, label, zone, min, max); }
 
-void PdUI::openFrameBox(char* label)
+void PdUI::openFrameBox(const char* label)
 {
   if (!path.empty()) path += "/";
   path += mangle(label);
 }
-void PdUI::openTabBox(char* label)
+void PdUI::openTabBox(const char* label)
 {
   if (!path.empty()) path += "/";
   path += mangle(label);
 }
-void PdUI::openHorizontalBox(char* label)
+void PdUI::openHorizontalBox(const char* label)
 {
   if (!path.empty()) path += "/";
   path += mangle(label);
 }
-void PdUI::openVerticalBox(char* label)
+void PdUI::openVerticalBox(const char* label)
 {
   if (!path.empty()) path += "/";
   path += mangle(label);
@@ -609,7 +617,7 @@ static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
       if (argc == 0) {
 	t_atom arg;
 	SETFLOAT(&arg, (float)x->active);
-	outlet_anything(x->out, gensym("active"), 1, &arg);
+	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
       } else if (argc == 1 &&
 		 (argv[0].a_type == A_FLOAT ||
 		  argv[0].a_type == A_DEFFLOAT)) {
@@ -692,16 +700,16 @@ extern "C" void faust_setup(mydsp)
     class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
 	      sizeof(t_faust), CLASS_DEFAULT,
 	      A_GIMME, A_NULL);
-  class_addmethod(faust_class, (t_method)faust_dsp, gensym("dsp"), A_NULL);
+  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
   class_addanything(faust_class, faust_any);
   class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
-  s_button = gensym("button");
-  s_checkbox = gensym("checkbox");
-  s_vslider = gensym("vslider");
-  s_hslider = gensym("hslider");
-  s_nentry = gensym("nentry");
-  s_vbargraph = gensym("vbargraph");
-  s_hbargraph = gensym("hbargrap");
+  s_button = gensym((char*)"button");
+  s_checkbox = gensym((char*)"checkbox");
+  s_vslider = gensym((char*)"vslider");
+  s_hslider = gensym((char*)"hslider");
+  s_nentry = gensym((char*)"nentry");
+  s_vbargraph = gensym((char*)"vbargraph");
+  s_hbargraph = gensym((char*)"hbargrap");
   /* give some indication that we're loaded and ready to go */
   mydsp dsp = mydsp();
   post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
